import botocore.session
import botocore_tornado.service


class AsyncSession(botocore.session.Session):
    def get_service(self, service_name, api_version=None):
        """
        Get information about a service.

        :type service_name: str
        :param service_name: The name of the service (e.g. 'ec2')

        :returns: :class:`botocore.service.Service`
        """
        service = botocore_tornado.service.get_service(self, service_name,
                                                       self.provider,
                                                       api_version=api_version)
        event = self.create_event('service-created')
        self._events.emit(event, service=service)
        return service


def get_session(env_vars=None):
    return AsyncSession(env_vars)



import logging

import botocore.service

from .endpoint import EndpointCreator
from .operation import AsyncOperation

logger = logging.getLogger(__name__)


class AsyncService(botocore.service.Service):

    def _create_operation_objects(self):
        logger.debug("Creating operation objects for: %s", self)
        operations = []
        for operation_name in self._operations_data:
            data = self._operations_data[operation_name]
            data['name'] = operation_name
            model = self._model.operation_model(operation_name)
            op = AsyncOperation(self, data, model)
            operations.append(op)
        return operations

    def get_endpoint(self, region_name=None, is_secure=True,
                     endpoint_url=None, verify=None):
        """
        Return the Endpoint object for this service in a particular
        region.

        :type region_name: str
        :param region_name: The name of the region.

        :type is_secure: bool
        :param is_secure: True if you want the secure (HTTPS) endpoint.

        :type endpoint_url: str
        :param endpoint_url: You can explicitly override the default
            computed endpoint name with this parameter.  If this arg is
            provided then neither ``region_name`` nor ``is_secure``
            is used in building the final ``endpoint_url``.
            ``region_name`` can still be useful for services that require
            a region name independent of the endpoint_url (for example services
            that use Signature Version 4, which require a region name for
            use in the signature calculation).

        """
        resolver = self.session.get_component('endpoint_resolver')
        region = self.session.get_config_variable('region')
        event_emitter = self.session.get_component('event_emitter')
        response_parser_factory = self.session.get_component(
            'response_parser_factory')
        credentials = self.session.get_credentials()
        user_agent = self.session.user_agent()
        endpoint_creator = EndpointCreator(resolver, region, event_emitter,
                                           credentials, user_agent)
        kwargs = {'service_model': self._model, 'region_name': region_name,
                  'is_secure': is_secure, 'endpoint_url': endpoint_url,
                  'verify': verify,
                  'response_parser_factory': response_parser_factory}
        if self._has_custom_signature_version:
            kwargs['signature_version'] = self.signature_version

        return endpoint_creator.create_endpoint(**kwargs)


def get_service(session, service_name, provider, api_version=None):
    """
    Return a Service object for a given provider name and service name.

    :type service_name: str
    :param service_name: The name of the service.

    :type provider: Provider
    :param provider: The Provider object associated with the session.
    """
    logger.debug("Creating service object for: %s", service_name)
    return AsyncService(session, provider, service_name)



import logging

import botocore.operation
from tornado import gen

logger = logging.getLogger(__name__)


class AsyncOperation(botocore.operation.Operation):

    @gen.coroutine
    def call(self, endpoint, **kwargs):
        from .endpoint import AsyncEndpoint
        assert isinstance(endpoint, AsyncEndpoint)
        # copied from botocore.operation.Operation.call
        logger.debug("%s called with kwargs: %s", self, kwargs)
        event = self.session.create_event('before-parameter-build',
                                          self.service.endpoint_prefix,
                                          self.name)
        self.session.emit(event, endpoint=endpoint,
                          model=self.model,
                          params=kwargs)
        request_dict = self.build_parameters(**kwargs)
        event = self.session.create_event('before-call',
                                          self.service.endpoint_prefix,
                                          self.name)
        # The operation kwargs is being passed in kwargs to support
        # handlers that still depend on this value.  Eventually
        # everything should move over to the model/endpoint args.
        self.session.emit(event, endpoint=endpoint,
                          model=self.model,
                          params=request_dict,
                          operation=self)

        response = yield endpoint.make_request(self.model, request_dict)

        event = self.session.create_event('after-call',
                                          self.service.endpoint_prefix,
                                          self.name)
        self.session.emit(event,
                          http_response=response[0],
                          model=self.model,
                          operation=self,
                          parsed=response[1])
        raise gen.Return(response)



import botocore.endpoint
from tornado import gen
from tornado.httpclient import AsyncHTTPClient, HTTPRequest


class AsyncEndpoint(botocore.endpoint.Endpoint):
    """Subclass of Endpoint that uses AsyncHTTPClient to make requests.
    The make_request method is wrapped in a coroutine"""

    def __init__(self, *args, **kwargs):
        super(AsyncEndpoint, self).__init__(*args, **kwargs)
        self.http_client = AsyncHTTPClient()

    @gen.coroutine
    def make_request(self, operation_model, request_dict):
        do_auth = self._signature_version and self.auth
        if do_auth:
            signer = self.auth
        else:
            signer = None
        request = self._create_request_object(request_dict)
        prepared_request = self.prepare_request(request, signer)
        body = prepared_request.body
        # handle the case when body is a file-like object. HTTPRequest expects
        # it to be a string, so we just read all of it here.
        if body is not None and hasattr(body, 'read'):
            body = body.read()
        # for some reason, the default transport (httplib) defaults to sending
        # an empty string as the body, instead of nothing, so here we check
        # if this is a PUT method and default the body to the empty string
        elif body is None and prepared_request.method == 'PUT':
            body = ''
        http_request = HTTPRequest(
            url=prepared_request.url, headers=prepared_request.headers,
            method=prepared_request.method, body=body)
        http_response = yield self.http_client.fetch(http_request)
        protocol = operation_model.metadata['protocol']
        response_dict = {
            'headers': http_response.headers,
            'status_code': http_response.code,
        }
        if response_dict['status_code'] >= 300:
            response_dict['body'] = http_response.body
        elif operation_model.has_streaming_output:
            response_dict['body'] = botocore.response.StreamingBody(
                http_response.body, response_dict['headers'].get('content-length'))
        else:
            response_dict['body'] = http_response.body
        parser = botocore.parsers.create_parser(protocol)
        parsed = parser.parse(
            response_dict, operation_model.output_shape)
        raise gen.Return((http_response, parsed))


class EndpointCreator(botocore.endpoint.EndpointCreator):
    def _get_endpoint(self, service_model, region_name, signature_version,
                      endpoint_url, verify, user_provided_creds,
                      response_parser_factory):
        """Copied from botocore.endpoint.EndpointCreator._get_endpoint
        so that the custom get_endpoint_complex can be used"""
        service_name = service_model.signing_name
        endpoint_prefix = service_model.endpoint_prefix
        credentials = self._credentials
        if user_provided_creds is not None:
            # Credentials provided via create_endpoint() override the
            # credentials provided when the EndpointCreator was created.
            credentials = user_provided_creds
        user_agent = self._user_agent
        event_emitter = self._event_emitter
        user_agent = self._user_agent
        return get_endpoint_complex(service_name, endpoint_prefix,
                                    signature_version, credentials,
                                    region_name, endpoint_url,
                                    verify, user_agent, event_emitter,
                                    response_parser_factory)


def get_endpoint(service, region_name, endpoint_url, verify=None):
    """Copied from botocore.endpoint.get_endpoint so that the custom
    get_endpoint_complex can be used"""
    service_name = getattr(service, 'signing_name', service.endpoint_prefix)
    endpoint_prefix = service.endpoint_prefix
    signature_version = getattr(service, 'signature_version', None)
    session = service.session
    credentials = session.get_credentials()
    event_emitter = session.get_component('event_emitter')
    user_agent = session.user_agent()
    return get_endpoint_complex(service_name, endpoint_prefix,
                                signature_version, credentials,
                                region_name, endpoint_url, verify, user_agent,
                                event_emitter)


def get_endpoint_complex(service_name, endpoint_prefix, signature_version,
                         credentials, region_name, endpoint_url, verify,
                         user_agent, event_emitter,
                         response_parser_factory=None):
    """Copied from botocore.endpoint.get_endpoint_complex so that the
    AsyncEndpoint can be returned"""
    auth = None
    if signature_version is not None:
        auth = botocore.endpoint._get_auth(signature_version,
                                           credentials=credentials,
                                           service_name=service_name,
                                           region_name=region_name)
    proxies = botocore.endpoint._get_proxies(endpoint_url)
    verify = botocore.endpoint._get_verify_value(verify)
    return AsyncEndpoint(
        region_name, endpoint_url, auth=auth,
        user_agent=user_agent,
        endpoint_prefix=endpoint_prefix,
        event_emitter=event_emitter,
        signature_version=signature_version,
        proxies=proxies,
        verify=verify,
        response_parser_factory=response_parser_factory)



#!/usr/bin/env python

"""
distutils/setuptools install script.
"""

import botocore_tornado

from setuptools import setup, find_packages


setup(
    name='botocore-tornado',
    version=botocore_tornado.__version__,
    description='botocore subclasses that uses AsyncHTTPClient',
    long_description=open('README.rst').read(),
    author='Simon Hewitt',
    author_email='simon@qudos.com.com',
    url='https://github.com/qudos-com/botocore-tornado',
    scripts=[],
    packages=find_packages(exclude=['tests*']),
    package_dir={'botocore_tornado': 'botocore_tornado'},
    include_package_data=True,
    install_requires=['botocore>=0.85.0',
                      'tornado>=4.0.0'],
    license=open("LICENSE.txt").read(),
    classifiers=(
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        # not tested on python3 yet!
        # 'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.3',
    ),
)
